import Vue from 'vue'
import App from './App.vue'
import router from './router'
import Vuex from 'vuex'
import Axios from 'axios';
import SystemInfoService from './api-services/system_info.service'
import BootstrapVue from 'bootstrap-vue'
import titleMixin from './mixins/title_mixin'
import isMobileMixin from './mixins/is_mobile_mixin'

import VueCtkDateTimePicker from 'vue-ctk-date-time-picker'
import 'vue-ctk-date-time-picker/dist/vue-ctk-date-time-picker.css'

import './icons'

import sharedMutations from 'vuex-shared-mutations'
import vuexUndoRedo from './mixins/vuex-undo-redo'

import APIUtil from './util/apiutil'
import WAPITransactionSerice from './api-services/transaction.service'
import LoginService from '@/api-services/login.service'

Vue.use(Vuex)
Vue.mixin(titleMixin)
Vue.mixin(isMobileMixin)
Vue.use(BootstrapVue)
Vue.use(vuexUndoRedo, {emptyStateMutation: 'emptyState'})

Vue.component('VueCtkDateTimePicker', VueCtkDateTimePicker)

let stored_token = JSON.parse(window.localStorage.getItem('token'))
if (stored_token != null) {
    stored_token = stored_token.token
}
let netdb_axios_config_tmp = {
    headers: {
        'Authorization': 'Bearer ' + stored_token,
        'Content-Type': 'application/json'
    }
}

//TODO: Refactor prototype definitions into their own file
Array.prototype.move = function (from, to) {
    this.splice(to, 0, this.splice(from, 1)[0]);
    return this;
}

const store = new Vuex.Store({
    state: {
        logged_in: window.localStorage.getItem('user') != null,
        user: JSON.parse(window.localStorage.getItem('user')),
        token: JSON.parse(window.localStorage.getItem('token')),
        ta_list: JSON.parse(window.localStorage.getItem('ta_list')),
        netdb_axios_config: netdb_axios_config_tmp,
        done: JSON.parse(window.localStorage.getItem('done')) || [],
        undone: JSON.parse(window.localStorage.getItem('undone')) || [],
        undo_redo_new_mutaion: JSON.parse(window.localStorage.getItem('undo_redo_new_mutation')) || true,
        transaction_result: JSON.parse(window.localStorage.getItem('transaction_result')) || null,
        executing_transaction: JSON.parse(window.localStorage.getItem('executing_transaction')) || false,
        show_sidebar: JSON.parse(window.localStorage.getItem('show_sidebar')) || false,
        reload_count: 0
    },
    mutations: {
        emptyState(state) {
            state.ta_list = null;
            window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
        },
        setUndoRedoNewMutation(state, payload) {
            state.undo_redo_new_mutaion = payload
            window.localStorage.setItem('undo_redo_new_mutation', JSON.stringify(state.undo_redo_new_mutaion))
        },
        popUndoRedoDone(state) {
            state.done.pop()
            window.localStorage.setItem('done', JSON.stringify(state.done))
        },
        popUndoRedoUndone(state) {
            state.undone.pop()
            window.localStorage.setItem('undone', JSON.stringify(state.undone))
        },
        pushUndoRedoDone(state, payload) {
            state.done.push(payload)
            window.localStorage.setItem('done', JSON.stringify(state.done))
        },
        pushUndoRedoUndone(state, payload) {
            state.undone.push(payload)
            window.localStorage.setItem('undone', JSON.stringify(state.undone))
        },
        clearUndoRedoDone(state) {
            state.done = []
            window.localStorage.setItem('done', JSON.stringify(state.done))
        },
        clearUndoRedoUndone(state) {
            state.undone = []
            window.localStorage.setItem('undone', JSON.stringify(state.undone))
        },
        login(state, payload) {
            let user = payload.user
            let token = payload.token
            state.logged_in = true
            state.user = user
            state.token = token
            window.localStorage.setItem('user', JSON.stringify(user))
            window.localStorage.setItem('token', JSON.stringify(token))
            state.netdb_axios_config = {
                headers: {
                    'Authorization': 'Bearer ' + token.token,
                    'Content-Type': 'application/json'
                }
            }
        },
        logout(state) {
            state.logged_in = false
            state.user = null
            state.token = null
            state.ta_list = []
            state.undo_redo_new_mutaion = true
            state.undone = []
            state.done = []
            state.transaction_result = null
            state.executing_transaction = false
            state.show_sidebar = false
            window.localStorage.removeItem('done')
            window.localStorage.removeItem('undone')
            window.localStorage.removeItem('undo_redo_new_mutation')
            window.localStorage.removeItem('user')
            window.localStorage.removeItem('token')
            window.localStorage.removeItem('ta_list')
            window.localStorage.removeItem('transaction_result')
            window.localStorage.removeItem('executing_transaction')
            window.localStorage.removeItem('show_sidebar')
            state.netdb_axios_config = {
                headers: {
                    'Authorization': null,
                    'Content-Type': 'application/json'
                }
            }
        },
        removeTransactionElement(state, payload) {
            for (let i = 0; i < state.ta_list.length; i++) {
                if (state.ta_list[i].uuid === payload.uuid) {
                    state.ta_list.splice(i, 1)
                    break
                }
            }
            window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
        },
        addTransactionElement(state, payload) {
            if (state.ta_list == null) {
                state.ta_list = []
            }
            let update = false
            for (let i = 0; i < state.ta_list.length; i++) {
                if (state.ta_list[i].uuid === payload.old_uuid) {
                    update = true
                    Vue.set(state.ta_list, i, payload)
                    break
                }
            }
            if (!update) {
                state.ta_list.push(payload)
            }
            window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
        },
        emptyTransactionList(state) {
            state.ta_list = []
            window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
        },
        async testTransaction(state) {
            this.commit('setTransactionBusy', true)

            let api_list = APIUtil.buildAPITaFromTaObjectArray(state.ta_list)
            try {
                let res = await WAPITransactionSerice.executeDry(state.netdb_axios_config, api_list)
                window.console.debug(res)
                state.transaction_result = {
                    'type': 'test-success'
                }
            } catch (e) {
                window.console.debug(e.response)
                state.transaction_result = {
                    'type': 'error',
                    'error': e.response.data.exception,
                    'uuid': state.ta_list[APIUtil.getAPIErrorIndexFromDBException(e.response.data.exception)].uuid
                }
            } finally {
                this.commit('setTransactionBusy', false)
                window.localStorage.setItem('transaction_result', JSON.stringify(state.transaction_result))
            }
        },
        showSidebar(state, show) {
            state.show_sidebar = show
            window.localStorage.setItem('show_sidebar', show)
        },
        setTransactionBusy(state, busy) {
            state.executing_transaction = busy
            window.localStorage.setItem('executing_transaction', busy)
        },
        removeTransactionResult(state) {
            state.transaction_result = null
            window.localStorage.removeItem('transaction_result')
        },
        reloadRouterComp(state) {
            state.reload_count++
        }
    },
    actions: {
        async executeTransaction({commit, state}) {
            commit('setTransactionBusy', true)

            let api_list = APIUtil.buildAPITaFromTaObjectArray(state.ta_list)
            try {
                let res = await WAPITransactionSerice.execute(state.netdb_axios_config, api_list)
                window.console.debug(res)
                state.ta_list = []
                state.undo_redo_new_mutaion = true
                state.undone = []
                state.done = []
                window.localStorage.removeItem('done')
                window.localStorage.removeItem('undone')
                window.localStorage.removeItem('undo_redo_new_mutation')
                window.localStorage.removeItem('ta_list')
                state.transaction_result = {
                    'type': 'success'
                }
            } catch (e) {
                window.console.debug(e.response)
                state.transaction_result = {
                    'type': 'error',
                    'error': e.response.data.exception,
                    'uuid': state.ta_list[APIUtil.getAPIErrorIndexFromDBException(e.response.data.exception)].uuid
                }
            } finally {
                commit('setTransactionBusy', false)
                window.localStorage.setItem('transaction_result', JSON.stringify(state.transaction_result))
            }
        },
    },
    plugins: [
        sharedMutations(
            {
                predicate:
                    ['pushUndoRedoUndone', 'popUndoRedoUndone', 'clearUndoRedoUndone', 'setUndoRedoNewMutation', 'clearUndoRedoDone', 'popUndoRedoDone', 'emptyState', 'login', 'logout', 'addTransactionElement', 'removeTransactionElement', 'emptyTransactionList', 'removeTransactionResult', 'setTransactionBusy']
            }
        )
    ],
})
// Undo-Redo-watched mutations
const watchedMutations = ['addTransactionElement', 'removeTransactionElement', 'emptyTransactionList']

Vue.config.errorHandler = function (err, vm, info) {
    // handle error
    // `info` is a Vue-specific error info, e.g. which lifecycle hook
    // the error was found in. Only available in 2.2.0+
    alert(err, info);
    window.console.log(err)
}

window.onerror = function (message, source, lineno, colno, error) {
    window.console.log(error)
}
Axios.interceptors.response.use(
    function (response) {
        return response;
    },
    async function (error) {
        // handle error
        if (error.response) {
            var stat = error.response.status
            switch (stat) {
                case 401:
                    await LoginService.logout(null)
                    store.commit('logout')
                    router.push('/login')
                    break;
                case 403:
                    await LoginService.logout(null)
                    store.commit('logout')
                    router.push('/login')
                    break;
                default:
                    break;
            }
        } else {
            alert(error)
        }
        return Promise.reject(error)
    });
Vue.config.productionTip = false
Axios.defaults.baseURL = '/';

SystemInfoService.getAll().then((sysinfo) => {
        Vue.prototype.$sysinfo = sysinfo.data
        let mods_by_name = {}
        for (var i = 0; i < sysinfo.data.mods.length; i++) {
            mods_by_name[sysinfo.data.mods[i].name] = sysinfo.data.mods[i]
        }
        Vue.prototype.$sysinfo_mods_by_name = mods_by_name
        Vue.prototype.$sysinfo_js_mods = sysinfo.data.js_mods
        let v = new Vue({
            router,
            store,
            render: h => h(App),
            beforeCreate: function () {
            }
        }).$mount('#app')
        if (v.$store) {
            v.$store.subscribe((mutation, state) => {
                if (watchedMutations.indexOf(mutation.type) >= 0) {
                    v.$store.commit('pushUndoRedoDone', mutation)
                    if (state.undo_redo_new_mutaion) {
                        window.console.debug("New mutation")
                        v.$store.commit('clearUndoRedoUndone')
                    }
                }
            });
        }
    }
)
