import Vue from 'vue'
import Router from 'vue-router'

Vue.use(Router)

export default new Router({
    mode: 'history',
    base: process.env.BASE_URL,
    routes: [
        {
            path: '/', beforeEnter: (to, from, next) => next('/dnsvs')
        },
        {
            path: '/swagger',
            name: 'wagger-ui',
            component: () => import('./views/Swagger.vue'),
            meta: {
                resolveName: function () {
                    return "API-Browser"
                }
            }
        },
        {
            path: '/login',
            name: 'login',
            component: () => import('./views/Login.vue'),
            meta: {
                resolveName: function () {
                    return "Login"
                }
            }
        },
        {
            path: '/oic_login',
            name: 'login_oic',
            component: () => import('./views/OICLogin.vue'),
            meta: {
                resolveName: function () {
                    return "OIC Login"
                }
            }
        },
        {
            path: '/user/tokens',
            name: 'tokens',
            component: () => import('./views/netdb/Tokens.vue'),
            meta: {
                resolveName: function () {
                    return "Accounts & Tokens"
                }
            }
        },
        {
            path: '/macauth',
            name: 'macauth',
            component: () => import('./views/macauth/MACAuth.vue'),
            meta: {
                resolveName: function () {
                    return "MACAuth"
                }
            }
        },
        {
            path: '/netdoc/map',
            name: 'map',
            component: () => import('./views/nd/Map.vue'),
            meta: {
                resolveName: function () {
                    return "Campusplan"
                }
            }
        },
        {
            path: '/dnsvs',
            component: () => import('./views/dnsvs/Main.vue'),
            meta: {
                resolveName: function () {
                    return "DNSVS"
                }
            },
            children: [
                {
                    path: '', beforeEnter: (to, from, next) => next('/dnsvs/bcds')
                },
                {
                    path: 'fqdns',
                    name: 'fqdns',
                    component: () => import('./views/dnsvs/Entry.vue'),
                    meta: {
                        resolveName: function () {
                            return "FQDNs"
                        }
                    }
                },
                {
                    path: 'fqdns/:fqdn',
                    name: 'DomainInspect',
                    component: () => import('./views/dnsvs/DomainInspect.vue'),
                    meta: {
                        resolveName: function (query) {
                            return (query.fqdn === '$') ? '.' : query.fqdn
                        },
                        resolveParents(query) {
                            let domains = query.fqdn.split('.').reverse()
                            window.console.log(domains)
                            let res = ['/dnsvs/fqdns']
                            let buf = ''
                            for (let i = 0; i < domains.length - 1; i++) {
                                let rbuf = ''
                                let f = domains[i]
                                if (f !== '' && f !== '$') {
                                    buf = f + '.' + buf
                                    rbuf = buf
                                } else {
                                    rbuf = '$'
                                }
                                res.unshift({'text': (rbuf === '$') ? '.' : rbuf, 'to': '/dnsvs/fqdns/' + rbuf})
                            }
                            window.console.log(res)
                            return res

                        }
                    }
                },
                {
                    path: 'fqdns/:fqdn/records',
                    name: 'fqdn_records',
                    component: () => import('./views/dnsvs/FQDNRecords.vue'),
                    meta: {
                        resolveName: function () {
                            return "Records"
                        },
                        resolveParents: function (query) {
                            return [`/dnsvs/fqdns/${query.fqdn}`]

                        }
                    }
                },
                {
                    path: 'fqdns/:fqdn/references',
                    name: 'fqdn_references',
                    component: () => import('./views/dnsvs/FQDNReferences.vue'),
                    meta: {
                        resolveName: function () {
                            return "Referenzen auf FQDN"
                        },
                        resolveParents: function (query) {
                            return [`/dnsvs/fqdns/${query.fqdn}`]

                        }
                    }
                },
                {
                    path: 'bcds',
                    name: 'bcds',
                    component: () => import('./views/dnsvs/Entry.vue'),
                    meta: {
                        resolveName: function () {
                            return "BCDs"
                        }
                    }
                },
                {
                    path: 'bcds/:name',
                    component: () => import('./views/dnsvs/BCDRecords.vue'),
                    meta: {
                        resolveName: function (query) {
                            return query.name
                        },
                        resolveParents: function () {
                            return ['/dnsvs/bcds']
                        }
                    }
                }
            ]
        },
        {
            path: '/dhcpvs',
            name: 'dhcpvs',
            component: () => import('./views/dhcpvs/main.vue'),
            meta: {
                resolveName: function () {
                    return "DHCPVS"
                }
            },
            children: [
                {
                    path: '', beforeEnter: (to, from, next) => next('/dhcpvs/ranges')
                },
                {
                    path: 'ranges',
                    name: 'ranges',
                    component: () => import('./views/dhcpvs/bcd_dhcp_overview.vue'),
                    meta: {
                        resolveName: function () {
                            return "Bereiche"
                        }
                    }
                },
                {
                    path: 'ranges/:name/',
                    component: () => import('./views/dhcpvs/bcd_dhcp_settings.vue'),
                    meta: {
                        resolveName: function (query) {
                            return query.name
                        },
                    }
                },
                {
                    path: 'activatedhcp/:name',
                    component: () => import('./views/dhcpvs/activate_dhcp'),
                    meta: {
                        resolveName: function (query) {
                            return query.name
                        },
                    }
                }
            ]
        }
    ]
})
