import ldap
from net_suite.model import SimpleKITUser
from ldap.filter import escape_filter_chars


class LDAP(object):
    def __init__(self, ldap_server, ldap_bind_user, ldap_bind_pw, default_search_base):
        self.ldap_server = ldap_server
        self.ldap_bind_user = ldap_bind_user
        self.ldap_bind_pw = ldap_bind_pw
        self.default_search_base = default_search_base

    def login(self, username, password, search_base=None):
        sb = search_base if search_base is not None else self.default_search_base
        con = ldap.initialize(self.ldap_server, bytes_mode=False)
        con.simple_bind_s(self.ldap_bind_user,
                          self.ldap_bind_pw)
        results = con.search_s(sb, ldap.SCOPE_SUBTREE, ldap.escape_str(escape_filter_chars, "(cn=%s)", username))
        try:
            if len(results) > 0:
                ldap_response = results[0]
                con.bind_s(ldap_response[0], password)
                con.unbind_s()
                return True

        except (ldap.INVALID_CREDENTIALS, ldap.UNWILLING_TO_PERFORM) as e:
            con.unbind_s()
        return False

    def get_simple_kit_user(self, username, search_base=None):
        sb = search_base if search_base is not None else self.default_search_base
        con = ldap.initialize(self.ldap_server, bytes_mode=False)
        con.simple_bind_s(self.ldap_bind_user,
                          self.ldap_bind_pw)
        results = con.search_s(sb, ldap.SCOPE_SUBTREE, ldap.escape_str(escape_filter_chars, "(cn=%s)", username))
        try:
            if len(results) > 0:
                ldap_response = results[0][1]
                con.unbind_s()
                return SimpleKITUser(login_name=ldap_response['cn'][0].decode('utf-8'),
                                     first_name=ldap_response['givenName'][0].decode('utf-8') if 'givenName' in ldap_response else None,
                                     last_name=ldap_response['sn'][0].decode('utf-8') if 'sn' in ldap_response else None,
                                     email=ldap_response['mail'][0].decode('utf-8'))

        except (ldap.INVALID_CREDENTIALS, ldap.UNWILLING_TO_PERFORM) as e:
            con.unbind_s()
        return None
